require "import"
import "android.widget.*"
import "android.view.*"
import "android.app.*"
import "android.os.Environment"
import "java.util.*"
import "java.io.*"

local activity = activity
local editText
local fileNameEditText
local headingEditText
local headingNumberEditText
local optionsSpinner
local boldCheckBox
local italicCheckBox
local underlineCheckBox
local imageButton
local colorButton
local fontSizeSpinner
local fontFamilySpinner

function onCreate(savedInstanceState)
    activity.setTitle("HTML Editor 1.4")
    local layout = LinearLayout(activity)
    layout.setOrientation(1)

    local scrollView = ScrollView(activity)
    local innerLayout = LinearLayout(activity)
    innerLayout.setOrientation(1)
    scrollView.addView(innerLayout)

    editText = EditText(activity)
    editText.setHint("Enter your HTML content here")
    editText.setTextSize(16)
    editText.setMinLines(5)
    editText.setVerticalScrollBarEnabled(true)
    editText.setHorizontalScrollBarEnabled(true)
    innerLayout.addView(editText)

    local optionsLayout = LinearLayout(activity)
    optionsLayout.setOrientation(0)

    local optionsLabel = TextView(activity)
    optionsLabel.setText("Insert: ")
    optionsLabel.setTextSize(16)
    optionsLayout.addView(optionsLabel)

    optionsSpinner = Spinner(activity)
    local options = {"Paragraph", "Heading", "Link", "Image", "Bold", "Italic", "Underline", "Horizontal Rule", "Ordered List", "Unordered List"}
    local adapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, options)
    adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
    optionsSpinner.setAdapter(adapter)
    optionsLayout.addView(optionsSpinner)

    local insertButton = Button(activity)
    insertButton.setText("Insert")
    insertButton.setOnClickListener{
        onClick = function(v)
            local selectedOption = options[optionsSpinner.getSelectedItemPosition() + 1]
            if selectedOption == "Paragraph" then
                insertParagraph()
            elseif selectedOption == "Heading" then
                showHeadingDialog()
            elseif selectedOption == "Link" then
                showLinkDialog()
            elseif selectedOption == "Image" then
                showImageDialog()
            elseif selectedOption == "Bold" then
                insertBold()
            elseif selectedOption == "Italic" then
                insertItalic()
            elseif selectedOption == "Underline" then
                insertUnderline()
            elseif selectedOption == "Horizontal Rule" then
                insertHorizontalRule()
            elseif selectedOption == "Ordered List" then
                insertOrderedList()
            elseif selectedOption == "Unordered List" then
                insertUnorderedList()
            end
        end
    }
    optionsLayout.addView(insertButton)

    innerLayout.addView(optionsLayout)

    local formattingLayout = LinearLayout(activity)
    formattingLayout.setOrientation(0)

    boldCheckBox = CheckBox(activity)
    boldCheckBox.setText("Bold")
    formattingLayout.addView(boldCheckBox)

    italicCheckBox = CheckBox(activity)
    italicCheckBox.setText("Italic")
    formattingLayout.addView(italicCheckBox)

    underlineCheckBox = CheckBox(activity)
    underlineCheckBox.setText("Underline")
    formattingLayout.addView(underlineCheckBox)

    imageButton = Button(activity)
    imageButton.setText("Insert Image")
    imageButton.setOnClickListener{
        onClick = function(v)
            showImageDialog()
        end
    }
    formattingLayout.addView(imageButton)

    colorButton = Button(activity)
    colorButton.setText("Change Color")
    colorButton.setOnClickListener{
        onClick = function(v)
            showColorDialog()
        end
    }
    formattingLayout.addView(colorButton)

    fontSizeSpinner = Spinner(activity)
    local fontSizes = {"8", "10", "12", "14", "16", "18", "20", "22", "24", "26", "28", "30"}
    local fontSizeAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, fontSizes)
    fontSizeAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
    fontSizeSpinner.setAdapter(fontSizeAdapter)
    formattingLayout.addView(fontSizeSpinner)

    fontFamilySpinner = Spinner(activity)
    local fontFamilies = {"Arial", "Times New Roman", "Georgia", "Verdana", "Courier New", "Comic Sans MS"}
    local fontFamilyAdapter = ArrayAdapter(activity, android.R.layout.simple_spinner_item, fontFamilies)
    fontFamilyAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
    fontFamilySpinner.setAdapter(fontFamilyAdapter)
    formattingLayout.addView(fontFamilySpinner)

    innerLayout.addView(formattingLayout)

    local fileLayout = LinearLayout(activity)
    fileLayout.setOrientation(0)

    local fileNameLabel = TextView(activity)
    fileNameLabel.setText("File Name: ")
    fileNameLabel.setTextSize(16)
    fileLayout.addView(fileNameLabel)

    fileNameEditText = EditText(activity)
    fileNameEditText.setHint("Enter file name")
    fileNameEditText.setTextSize(16)
    fileLayout.addView(fileNameEditText)

    local saveButton = Button(activity)
    saveButton.setText("Save HTML")
    saveButton.setOnClickListener{
        onClick = function(v)
            saveHTMLToFile()
        end
    }
    fileLayout.addView(saveButton)

    innerLayout.addView(fileLayout)

    local donationMessage = TextView(activity)
    donationMessage.setText("If you find this app useful, please consider making a donation to support its development You can add mobile money to developers. Top up your mobile phone to number 0933967101.")
    donationMessage.setTextSize(14)
    donationMessage.setGravity(Gravity.CENTER)
    innerLayout.addView(donationMessage)

    activity.setContentView(scrollView)
end

function insertParagraph()
    editText.getText().append("<p></p>")
end

function insertBold()
    local selectedText = editText.getText().subSequence(editText.getSelectionStart(), editText.getSelectionEnd()).toString()
    local boldText = "<b>" .. selectedText .. "</b>"
    editText.getText().replace(editText.getSelectionStart(), editText.getSelectionEnd(), boldText)
end

function insertItalic()
    local selectedText = editText.getText().subSequence(editText.getSelectionStart(), editText.getSelectionEnd()).toString()
    local italicText = "<i>" .. selectedText .. "</i>"
    editText.getText().replace(editText.getSelectionStart(), editText.getSelectionEnd(), italicText)
end

function insertUnderline()
    local selectedText = editText.getText().subSequence(editText.getSelectionStart(), editText.getSelectionEnd()).toString()
    local underlineText = "<u>" .. selectedText .. "</u>"
    editText.getText().replace(editText.getSelectionStart(), editText.getSelectionEnd(), underlineText)
end

function insertHorizontalRule()
    editText.getText().append("<hr>")
end

function insertOrderedList()
    editText.getText().append("<ol><li></li></ol>")
end

function insertUnorderedList()
    editText.getText().append("<ul><li></li></ul>")
end

function showHeadingDialog()
    local headingDialog = AlertDialog.Builder(activity)
    headingDialog.setTitle("Insert Heading")

    local layout = LinearLayout(activity)
    layout.setOrientation(1)

    local headingLabel = TextView(activity)
    headingLabel.setText("Heading Text:")
    layout.addView(headingLabel)

    headingEditText = EditText(activity)
    layout.addView(headingEditText)

    local headingNumberLabel = TextView(activity)
    headingNumberLabel.setText("Heading Number:")
    layout.addView(headingNumberLabel)

    headingNumberEditText = EditText(activity)
    layout.addView(headingNumberEditText)

    headingDialog.setView(layout)
    headingDialog.setPositiveButton("Insert", {
        onClick = function(dialog, which)
            local headingText = headingEditText.getText().toString()
            local headingNumber = tonumber(headingNumberEditText.getText().toString())
            if headingNumber and headingNumber > 0 and headingNumber <= 6 then
                local htmlTag = "<h" .. tostring(headingNumber) .. ">" .. headingText .. "</h" .. tostring(headingNumber) .. ">"
                editText.getText().append(htmlTag)
            else
                Toast.makeText(activity, "Please enter a valid heading number (1-6).", Toast.LENGTH_SHORT).show()
            end
            dialog.dismiss()
        end
    })
    headingDialog.setNegativeButton("Cancel", {
        onClick = function(dialog, which)
            dialog.dismiss()
        end
    })
    headingDialog.show()
end

function showLinkDialog()
    local linkDialog = AlertDialog.Builder(activity)
    linkDialog.setTitle("Insert Link")

    local layout = LinearLayout(activity)
    layout.setOrientation(1)

    local linkTextLabel = TextView(activity)
    linkTextLabel.setText("Link Text:")
    layout.addView(linkTextLabel)

    local linkTextEditText = EditText(activity)
    layout.addView(linkTextEditText)

    local linkSourceLabel = TextView(activity)
    linkSourceLabel.setText("Link Source:")
    layout.addView(linkSourceLabel)

    local linkSourceEditText = EditText(activity)
    layout.addView(linkSourceEditText)

    linkDialog.setView(layout)
    linkDialog.setPositiveButton("Insert", {
        onClick = function(dialog, which)
            local linkText = linkTextEditText.getText().toString()
            local linkSource = linkSourceEditText.getText().toString()
            if linkText ~= "" and linkSource ~= "" then
                local htmlLink = "<a href=\"" .. linkSource .. "\">" .. linkText .. "</a>"
                editText.getText().append(htmlLink)
            else
                Toast.makeText(activity, "Please enter both link text and link source.", Toast.LENGTH_SHORT).show()
            end
            dialog.dismiss()
        end
    })
    linkDialog.setNegativeButton("Cancel", {
        onClick = function(dialog, which)
            dialog.dismiss()
        end
    })
    linkDialog.show()
end

function showImageDialog()
    local imageDialog = AlertDialog.Builder(activity)
    imageDialog.setTitle("Insert Image")

    local layout = LinearLayout(activity)
    layout.setOrientation(1)

    local imageSourceLabel = TextView(activity)
    imageSourceLabel.setText("Image Source:")
    layout.addView(imageSourceLabel)

    local imageSourceEditText = EditText(activity)
    layout.addView(imageSourceEditText)

    local imageAltLabel = TextView(activity)
    imageAltLabel.setText("Image Alt Text:")
    layout.addView(imageAltLabel)

    local imageAltEditText = EditText(activity)
    layout.addView(imageAltEditText)

    imageDialog.setView(layout)
    imageDialog.setPositiveButton("Insert", {
        onClick = function(dialog, which)
            local imageSource = imageSourceEditText.getText().toString()
            local imageAlt = imageAltEditText.getText().toString()
            if imageSource ~= "" then
                local htmlImage = "<img src=\"" .. imageSource .. "\" alt=\"" .. imageAlt .. "\">"
                editText.getText().append(htmlImage)
            else
                Toast.makeText(activity, "Please enter the image source.", Toast.LENGTH_SHORT).show()
            end
            dialog.dismiss()
        end
    })
    imageDialog.setNegativeButton("Cancel", {
        onClick = function(dialog, which)
            dialog.dismiss()
        end
    })
    imageDialog.show()
end

function showColorDialog()
    local colorDialog = AlertDialog.Builder(activity)
    colorDialog.setTitle("Change Text Color")

    local layout = LinearLayout(activity)
    layout.setOrientation(1)

    local colorLabel = TextView(activity)
    colorLabel.setText("Enter color (e.g., #FF0000 for red):")
    layout.addView(colorLabel)

    local colorEditText = EditText(activity)
    layout.addView(colorEditText)

    colorDialog.setView(layout)
    colorDialog.setPositiveButton("Apply", {
        onClick = function(dialog, which)
            local color = colorEditText.getText().toString()
            local selectedText = editText.getText().subSequence(editText.getSelectionStart(), editText.getSelectionEnd()).toString()
            local coloredText = "<span style=\"color:" .. color .. ";\">" .. selectedText .. "</span>"
            editText.getText().replace(editText.getSelectionStart(), editText.getSelectionEnd(), coloredText)
            dialog.dismiss()
        end
    })
    colorDialog.setNegativeButton("Cancel", {
        onClick = function(dialog, which)
            dialog.dismiss()
        end
    })
    colorDialog.show()
end

function saveHTMLToFile()
    local htmlContent = editText.getText().toString()
    local fileName = fileNameEditText.getText().toString()

    if fileName == "" then
        fileName = "random_" .. tostring(math.random(1000, 9999)) .. ".html"
    end

    local file = File(Environment.getExternalStorageDirectory().getAbsolutePath() .. "/" .. fileName)
    local writer = BufferedWriter(FileWriter(file))
    writer.write(htmlContent)
    writer.flush()
    writer.close()

    local storageDir = Environment.getExternalStorageDirectory().getAbsolutePath()
    Toast.makeText(activity, "HTML saved to file: " .. fileName .. "\nFile stored in: " .. storageDir, Toast.LENGTH_LONG).show()
end